package com.example.educheckin.controller;

import com.example.educheckin.annotation.Scope;
import com.example.educheckin.annotation.UserToken;
import com.example.educheckin.config.Token;
import com.example.educheckin.entity.User;
import com.example.educheckin.service.UserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.ArraySchema;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.ExampleObject;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@RestController
@Tag(name = "账户")
@ApiResponses({
        @ApiResponse(responseCode = "200", description = "请求成功", content = {@Content(schema = @Schema())}),
        @ApiResponse(responseCode = "400", description = "客户端错误", content = {@Content(schema = @Schema())}),
        @ApiResponse(responseCode = "401", description = "权限不足", content = {@Content(schema = @Schema())}),
        @ApiResponse(responseCode = "500", description = "服务器错误", content = {@Content(schema = @Schema())})})
public class UserController {
    @Autowired
    private UserService userService;

    @PutMapping("/register")
    @Operation(summary = "注册", description = "注册新用户")
    @ApiResponse(responseCode = "200", content = {@Content(schema = @Schema(implementation = User.class))})
    @ApiResponse(responseCode = "280", description = "邮箱不存在", content = {@Content(schema = @Schema(implementation = String.class), examples = @ExampleObject(value = "Email not exist."))})
    @ApiResponse(responseCode = "281", description = "邀请码不存在", content = {@Content(schema = @Schema(implementation = String.class), examples = @ExampleObject(value = "Invite code not exist."))})
    @ApiResponse(responseCode = "282", description = "邀请码已使用", content = {@Content(schema = @Schema(implementation = String.class), examples = @ExampleObject(value = "Invite code used."))})
    public ResponseEntity<String> register(@Parameter(description = "邮箱") @RequestParam String email,
                                           @Parameter(description = "密码") @RequestParam String password,
                                           @Parameter(description = "姓名") @RequestParam String name,
                                           @Parameter(description = "性别，为F或者M") @RequestParam String gender,
                                           @Parameter(description = "手机") @RequestParam String phone,
                                           @Parameter(description = "岗位") @RequestParam(required = false)
                                           String internship,
                                           @Parameter(description = "邀请码") @RequestParam String inviteCode) {
        User user = new User(email, password, name, gender, phone, internship, inviteCode);
        String ans = userService.register(user);
        return switch (ans) {
            case "success" -> ResponseEntity.status(200).body(ans);
            case "email exist" -> ResponseEntity.status(280).body(ans);
            case "invite code not exist" -> ResponseEntity.status(281).body(ans);
            case "code used" -> ResponseEntity.status(282).body(ans);
            default -> ResponseEntity.status(500).body(ans);
        };

    }

    @PostMapping("/login")
    @Operation(summary = "登录", description = "登录返回token")
    @ApiResponse(responseCode = "200", content = {@Content(schema = @Schema(implementation = String.class), examples = @ExampleObject(value = "Token"))})
    @ApiResponse(responseCode = "280", description = "登录失败", content = {@Content(schema = @Schema(implementation = String.class), examples = @ExampleObject(value = "Login failed."))})
    public ResponseEntity<String> login(@Parameter(description = "邮箱") @RequestParam String email,
                                        @Parameter(description = "密码") @RequestParam String password) {
        if (userService.login(email, password)) {
            String token = Token.getToken(userService.getUser(email));
            return ResponseEntity.status(200).body(token);
        }
        return ResponseEntity.status(280).body("Login failed.");
    }

    @PostMapping("/modify")
    @UserToken()
    @Scope()
    @Operation(summary = "修改信息", description = "修改用户信息，不修改的项传空值，需要token验证，只能修改自己的信息")
    @ApiResponse(responseCode = "200", content = {@Content(schema = @Schema(implementation = User.class))})
    @ApiResponse(responseCode = "280", description = "原有邮箱不存在", content = {@Content(schema = @Schema(implementation = String.class), examples = @ExampleObject(value = "Email not exist."))})
    //@ApiResponse(responseCode = "281", description = "修改后的邮箱已存在", content = {@Content(schema = @Schema(implementation = String.class), examples = @ExampleObject(value = "New email exist."))})
    public ResponseEntity<String> modify(@Parameter(description = "邮箱") @RequestParam String email,
                                         //@Parameter(description = "修改后的邮箱") @RequestParam(required = false) String newEmail,
                                         @Parameter(description = "修改后的密码") @RequestParam(required = false) String newPassword,
                                         @Parameter(description = "修改后的姓名") @RequestParam(required = false) String newName,
                                         @Parameter(description = "修改后的性别，为F或者M") @RequestParam(required = false) String newGender,
                                         @Parameter(description = "修改后的手机") @RequestParam(required = false) String newPhone,
                                         @Parameter(description = "修改后的岗位") @RequestParam(required = false)  String newInternship){
                                         //@Parameter(description = "修改后的组别") @RequestParam(required = false) List<String> newGroup,
                                         //@Parameter(description = "修改后的邀请码") @RequestParam(required = false) String newInviteCode) {
        User currentUser = userService.getUser(email);

        if (currentUser == null) {
            return ResponseEntity.status(280).body("Email not exist.");
        }

        User user = new User(email ,
                (newPassword == null)|(Objects.equals(newPassword, "")) ? currentUser.getPassword() : newPassword,
                (newName == null)|(Objects.equals(newName, "")) ? currentUser.getName() : newName,
                (newGender == null)|(Objects.equals(newGender, ""))  ? currentUser.getGender() : newGender,
                (newPhone == null)|(Objects.equals(newPhone, "")) ? currentUser.getPhone() : newPhone,
                (newInternship == null)|(Objects.equals(newInternship, "")) ? currentUser.getInternship() : newInternship,
                currentUser.getInviteCode(),
                currentUser.getGroup(),
                currentUser.getAdmin()
                );

        String ans = userService.changeInfo(email, user);

        return switch (ans) {
            case "success" -> ResponseEntity.status(200).body(ans);
            case "email not exist" -> ResponseEntity.status(280).body(ans);
            case "email exist" -> ResponseEntity.status(281).body(ans);
            default -> ResponseEntity.status(500).body(ans);
        };
    }

    @GetMapping("/getUser")
    @UserToken()
    @Scope(1)
    @Operation(summary = "获得用户信息", description = "获得用户信息, 需要token验证, 只能本人访问和大于等于1级权限的访问")
    @ApiResponse(responseCode = "200", content = {@Content(schema = @Schema(implementation = User.class))})
    @ApiResponse(responseCode = "280", description = "原有邮箱不存在", content = {@Content(schema = @Schema(implementation = String.class), examples = @ExampleObject(value = "Email not exist."))})
    public ResponseEntity<User> getUser(String email) {
        User user = userService.getUser(email);
        if (user == null) {
            return ResponseEntity.status(280).body(null);
        }
        return ResponseEntity.status(200).body(user);
    }

    @GetMapping("/getStudentList")
    @UserToken(1)
    @Scope(2)
    @Operation(summary = "获得所管理的学生列表", description = "获得所管理的学生列表, 需要token验证, 只能本人访问和大于等于2级权限的访问")
    @ApiResponse(responseCode = "200", content = @Content(array = @ArraySchema(schema = @Schema(implementation = User.class))))
    @ApiResponse(responseCode = "280", description = "用户不存在", content = {@Content(schema = @Schema(implementation = String.class), examples = @ExampleObject(value = "Email not exist."))})
    public ResponseEntity<List<User>> getStudentList(String email) {
        User user = userService.getUser(email);
        if (user == null) {
            return ResponseEntity.status(280).body(null);
        }
        List<String> group = user.getGroup();
        List<User> studentList = new ArrayList<>();
        for (String s : group) {
            studentList.addAll(userService.belongToGroup(s));
        }
        return ResponseEntity.status(200).body(studentList);
    }

    @GetMapping("/getGroupList")
    @Operation(summary = "获得组列表", description = "获得对应小组成员的列表")
    @ApiResponse(responseCode = "200", content = @Content(array = @ArraySchema(schema = @Schema(implementation = User.class))))
    @ApiResponse(responseCode = "280", description = "用户不存在", content = {@Content(schema = @Schema(implementation = String.class), examples = @ExampleObject(value = "Email not exist."))})
    public ResponseEntity<List<User>> getGroupList(String email) {
        User user = userService.getUser(email);
        if (user == null) {
            return ResponseEntity.status(280).body(null);
        }
        List<String> group = user.getGroup();
        List<User> studentList = new ArrayList<>();
        for (String s : group) {
            studentList.addAll(userService.belongToGroup(s));
        }
        for (User s: studentList) {
            if (s.getEmail().equals(email)) {
                studentList.remove(s);
                break;
            }
        }

        return ResponseEntity.status(200).body(studentList);
    }

    @PutMapping("/advise")
    @Operation(summary = "建议", description = "建议")
    @UserToken(-1)
    public ResponseEntity<Void> advise(@Parameter(description = "建议") @RequestParam String advise) {
        userService.advise(advise);
        return ResponseEntity.status(200).body(null);
    }

}
